// This handles turning super.
rawset(_G, "become_super", function(player)
	// Sanity check
	if player.can_super ~= true return end
	if player.superanim ~= 0    return end

	local canuse = true

	// Failure conditions!
	// Cannot turn super in enclosed spaces
	if player.mo.ceilingz - player.mo.floorz < P_GetPlayerHeight(player)
		canuse = false
	end

	// Error sound
	if not canuse
		return
	end

	player.charflags = $1 | SF_SUPER
	player.active_supertrans = true
end)

//Adding the ability to use Custom 3 to transform and detransform
addHook("ThinkFrame", do
	for player in players.iterate
		if player.mo and player.mo.skin == "modernsonic"
			if player.cmd.buttons & BT_CUSTOM3
			and not (player.powers[pw_super])
			and not (player.mo.state >= S_PLAY_SUPER_TRANS2 and player.mo.state <= S_PLAY_SUPER_TRANS6)
			and player.rings >= 50
			and player.custom3down == false
			and player.abilitylock == false
			and player.playerstate == PST_LIVE
			and not (P_PlayerInPain(player))
			and not (player.powers[pw_carry] == CR_NIGHTSMODE)
			and not player.exiting
				player.custom3down = true
				if All7Emeralds(emeralds)
					player.can_super = true
					become_super(player)
				end
			end
			if player.custom3down == false
			and player.powers[pw_super]
			and player.cmd.buttons & BT_CUSTOM3
			and player.pflags & PF_FINISHED
				player.powers[pw_super] = 0
				player.custom3down = true
				player.mo.color = player.skincolor
				player.superanim = 0
				player.mo.state = $
				P_SpawnShieldOrb(player)
				P_FlashPal(player, PAL_MIXUP, 7)
				S_StartSound(player.mo, sfx_s3k66)
				P_RestoreMusic(player)
				player.mo.color = player.skincolor
				player.can_super = false
			end
			if not (player.cmd.buttons & BT_CUSTOM3)
				player.custom3down = false
			end
			if player.active_supertrans == true
				A_PreSuper(player.mo)
			end
			if player.playerstate == PST_DEAD
				player.active_supertrans = false
			end
			if player.powers[pw_super] == 0
				player.superboost = false
			end

		end
	end
end)

//I'll be honest, I don't know how this works. But let's just be glad it does
rawset(_G, "SpawnMobjFrom", function(mobj, xofs, yofs, zofs, newtype)
	xofs = FixedMul(xofs, mobj.scale)
	yofs = FixedMul(yofs, mobj.scale)
	zofs = FixedMul(zofs, mobj.scale)

	local newobj = P_SpawnMobj(mobj.x+xofs, mobj.y+yofs, mobj.z+zofs, newtype)
	if (mobj.eflags & MFE_VERTICALFLIP)
		local elementheight = FixedMul(newobj.info.height, mobj.scale)

		newobj.eflags = $1 | MFE_VERTICALFLIP
		newobj.flags2 = $1 | MF2_OBJECTFLIP
		newobj.z = mobj.z + mobj.height - zofs - elementheight
	end
	newobj.destscale = mobj.destscale
	P_SetScale(newobj, mobj.scale)
	return newobj
end)

// This function handles the Super Sonic animation!
function A_PreSuper(actor, var1, var2)
	if not actor.player return end
	local player = actor.player

	local ourskin = "modernsonic"
	if player.skin ourskin = player.skin end

	//P_ResetPlayer(player)
	player.superanim = $1 + 1

	if player.superanim == 41
		if player.rings >= 50
			// We're officially "super" now
			player.charflags = $1 | SF_SUPER
			player.powers[pw_super] = $1+1
			player.powers[pw_extralife] = 0
			player.powers[pw_sneakers] = 0
			player.powers[pw_invulnerability] = 0
			S_StartSound(NULL, sfx_supert)
			//S_ChangeMusic("_SUPER", true, player)
			P_FlashPal(player, PAL_WHITE, 2)
			if player.mo.state == S_PLAY_WAIT
				player.mo.state = S_PLAY_STND
			elseif player.mo.state >= S_PLAY_SUPER_TRANS1 and player.mo.state <= S_PLAY_SUPER_TRANS6
				player.mo.state = S_PLAY_STND
			else
				player.mo.state = $
			end
			
			return
		else
			P_FlashPal(player, PAL_MIXUP, 2)
			S_StartSound(player.mo, sfx_s3k66)
		end
	elseif player.superanim > 41
		player.mo.state = $
		player.active_supertrans = false
		player.superanim = 0
		player.charability2 = skins[ourskin].ability2
		return
	end

	// Emeralds rotate around you
	local distsin = sin(FixedAngle((player.superanim*FRACUNIT/41)*180))
	local distance = 16*FRACUNIT + distsin*40

	local playerva = player.cmd.angleturn
	if ANGLE_90 > 16384 playerva = $1 << 16 end

	local angle, xofs, yofs, zofs, obj
	for i = 0, 6
		angle = (i * (103*FRACUNIT/2)) + player.superanim*20*FRACUNIT
		angle = FixedAngle(angle)
		xofs = FixedMul(sin(angle), distance)
		yofs = FixedMul(cos(angle), distance)
		zofs = FixedMul(cos(angle + (playerva + ANGLE_270)), distance>>2) + actor.height>>1
		obj = SpawnMobjFrom(actor, xofs, yofs, zofs, MT_FAKEEMMY)
		obj.frame = i | FF_FULLBRIGHT

		if player.superanim > 1
			obj.tics = 1
		end
	end
end

addHook("ThinkFrame", do
	for player in players.iterate
		if (player.mo and player.mo.skin == "modernsonic")
			if player.powers[pw_super]
			and player.playerstate == PST_LIVE
			and not (player.powers[pw_extralife])
			and not (player.pflags & PF_FINISHED)
				S_ChangeMusic("_SUPER", true, player)
			end
		end
	end
end)